package nrs.jester;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.table.*;

/**
 *
 * @author  NSYDENHA
 */
public class DriverPropertiesDialog extends JDialog implements ActionListener
{
    private JTable propertyTable = null;
    private DefaultTableModel dtm = null;
    private int returnStatus = -1;
    private JPopupMenu connMenu = null;
    public final static int OK = 0;
    public final static int CANCEL = -1;
    
    /** 
     * Creates a new instance of DriverPropertiesDialog
     * @param parent owning dialog window
     * @param driverProps Map of existing driver properties
     * @param connProps array of optional connection parameters
     */
    public DriverPropertiesDialog(JDialog parent, Properties driverProps, String[] connProps)
    {
	super(parent, "Driver properties", true);
	
	// add existing data to table
	String[][] rowData = new String[1][2];
	if (driverProps != null && driverProps.size() > 0) {
	    rowData = new String[driverProps.size()][2];
	    String key, value;
	    int row = 0;
	    Iterator it = driverProps.keySet().iterator();
	    while (it.hasNext()) {
		key = (String)it.next();
		value = driverProps.getProperty(key);
		rowData[row][0] = key;
		rowData[row++][1] = value;
	    }
	}
	// set up table
	String[] columnNames = {"Property", "Value"};
	dtm = new DefaultTableModel(rowData, columnNames);
	propertyTable = new JTable(dtm);
	propertyTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	propertyTable.setToolTipText("Right click to see connection properties");
	// set up pop-up menu of connection properties
	connMenu = new JPopupMenu();
	JMenuItem popMI;
	for (int i=0; i<connProps.length; i++) {
	    popMI = new JMenuItem(connProps[i]);
	    popMI.addActionListener(this);
	    connMenu.add(popMI);
	}
	propertyTable.addMouseListener(new MouseAdapter() {
	    public void mousePressed(MouseEvent me) {
		if ((me.getModifiers() & InputEvent.BUTTON3_MASK) == InputEvent.BUTTON3_MASK)
		{
		    connMenu.show(propertyTable, me.getX(), me.getY());
		}
	    }
	});
	JScrollPane tableScroll = new JScrollPane(propertyTable);
	
	Container content = getContentPane();
	GridBagLayout gbl = new GridBagLayout();
	GridBagConstraints gbc = new GridBagConstraints();
	content.setLayout(gbl);
	
	gbc.gridwidth = 2;
	gbc.gridheight = 2;
	gbc.fill = GridBagConstraints.BOTH;
	gbc.insets = new Insets(5, 5, 5, 5);
	gbl.setConstraints(tableScroll, gbc);
	content.add(tableScroll);
	
	// right of table
	JButton add = new JButton("Add");
	add.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		Object[] data = {null, null};
		dtm.addRow(data);
	    }
	});
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	gbc.gridheight = 1;
	gbc.insets = new Insets(5, 0, 5, 5);
	gbl.setConstraints(add, gbc);
	content.add(add);
	
	JButton delete = new JButton("Delete");
	delete.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		int selectedRow = propertyTable.getSelectedRow();
		if (selectedRow >= 0) dtm.removeRow(selectedRow);
	    }
	});
	gbc.anchor = GridBagConstraints.NORTH;
	gbl.setConstraints(delete, gbc);
	content.add(delete);
	
	// bottom of table
	JButton ok = new JButton("OK");
	ok.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		returnStatus = OK;
		setVisible(false);
	    }
	});
	gbc.gridwidth = 1;
	gbc.gridheight = 1;
	gbc.anchor = GridBagConstraints.WEST;
	gbc.fill = GridBagConstraints.NONE;
	gbc.insets = new Insets(0, 5, 5, 0);
	gbl.setConstraints(ok, gbc);
	content.add(ok);
	
	JButton cancel = new JButton("Cancel");
	cancel.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		returnStatus = CANCEL;
		setVisible(false);
	    }
	});
	gbc.gridwidth = 2;
	gbl.setConstraints(cancel, gbc);
	content.add(cancel);
	
	pack();
	setLocationRelativeTo(parent);
	setVisible(true);
    }
    
    /**
     * Get driver properties
     * @return driver properties
     */
    public Properties getDriverProperties()
    {
	String prop, value;
	Properties driverProps = new Properties();
	
	TableModel tm = propertyTable.getModel();
	for (int i=0; i<tm.getRowCount(); i++) {
	    prop = (String)tm.getValueAt(i, 0);
	    value = (String)tm.getValueAt(i, 1);
	    
	    if (prop != null && value != null) {
		driverProps.setProperty(prop, value);
	    }
	}
	
	return driverProps;
    }
    
    /** 
     * Get the return status from this dialog.
     * OK or CANCEL
     */
    public int getReturnStatus() {
	return returnStatus;
    }
    
    /** User selected an option from a pop-up menu */
    public void actionPerformed(ActionEvent e) {
	String cp = e.getActionCommand();
	int selRow = propertyTable.getSelectedRow();
	if (selRow > -1) dtm.setValueAt(cp, selRow, 0);
    }
}
