package nrs.jester;

import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.event.*;
import java.awt.print.*;
import java.io.*;
import java.net.*;
import java.sql.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import org.xml.sax.*;
import org.xml.sax.helpers.*;

/*
 * Java Stored Procedure Tester (Jester)
 * Copyright (C) 2002 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/**
 * Java Stored Procedure Tester (Jester)
 *
 * @author Nicholas Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class Jester extends JFrame implements Observer
{
    public final static String version = "1.0";
    private final static String CONFIG_FILE = ".jester/config.txt";

    private File spDoc = null;
    private Results results = null;
    private StoredProcedure sp = null;
    private SPReader spReader = new SPReader();
    private Map defaultValues = new HashMap();
    private Thread queryThread = null;
    private boolean runningQuery = false;
    private SPInputPanel inputs = null;
    private QueryPanel queryPanel = null;

    // settings
    private String dateFormat = "dd/MM/yyyy";
    private String timeFormat = "HH:mm";
    /** JDBC query timeout */
    private int timeout = 0;

    // Conection details object
    private DBConnection dbc = null;
    private ConnectionSetting currentConnection = null;

    // UI elements
    private JTextArea status = null;
    private PrintTable resultsTable = null;
    private JScrollPane resultsScroll = null;
    private JSplitPane spPanel = null;
    private JTextArea mdArea = null;
    private JScrollPane inputScroll = null;
    private JMenu dateMenu=null, timeMenu=null, timeoutMenu=null;
    private JMenuItem connectMI=null, disconnectMI=null;
    private JMenuItem csvMI=null, getSPMI=null;
    private JTabbedPane tabbedPane = null;
    private JDialog helpDialog=null, fieldDialog=null;
    private JTextField filter = null;
    private JRadioButton conRB = null;
    private JButton cancelButton = null;

    /** Icon locations */
    public final static String APP_ICON = "/pics/jester.png";
    public final static String GO_ICON = "/pics/go.png";
    public final static String GREEN_ICON = "/pics/greendot.png";
    public final static String OPEN_ICON = "/pics/open.png";
    public final static String RED_ICON = "/pics/reddot.png";
    public final static String SAVE_ICON = "/pics/save.png";
    public final static String STOP_ICON = "/pics/stop.png";

    /** ActionListener for query timeout menu items */
    private class SecondListener implements ActionListener
    {
	public void actionPerformed(ActionEvent e) {
	    JMenuItem mi = (JMenuItem)e.getSource();
	    String time = mi.getText();
	    // remove last s
	    timeout = Integer.parseInt(time.substring(0, time.length()-1));
	}
    }

    /**
     * Popup menu for table
     */
    private class CopyMenu extends MouseAdapter
    {
	public void mousePressed(MouseEvent e)
	{
	    if ((e.getModifiers() & MouseEvent.BUTTON3_MASK) == MouseEvent.BUTTON3_MASK)
	    {
		JPopupMenu popup = new JPopupMenu();
		JMenuItem copy = new JMenuItem("Copy");
		copy.addActionListener(new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
			String val = (String)results.getValueAt(resultsTable.getSelectedRow(),
			    resultsTable.getSelectedColumn());
			Clipboard cb = Toolkit.getDefaultToolkit().getSystemClipboard();
			StringSelection contents = new StringSelection(val);
			cb.setContents(contents, null);
		    }
		});
		popup.add(copy);

		JMenuItem mi;
		int tabIndex = tabbedPane.getSelectedIndex();
		if (tabIndex == 0) // SP panel
		{
		    // option to copy to in parameter
		    if (sp != null) {
			java.util.List spList = sp.getInParameters();
			Iterator it = spList.iterator();
			Parameter param;
			while (it.hasNext()) {
			    param = (Parameter)it.next();
			    mi = new JMenuItem("Copy to "+param.getDescription());
			    mi.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
				    String val = (String)results.getValueAt(resultsTable.getSelectedRow(),
					resultsTable.getSelectedColumn());
				    String paramDesc = e.getActionCommand().substring(8);
				    inputs.setInputValue(val, paramDesc);
				}
			    });
			    popup.add(mi);
			}
		    }
		}
		else if (tabIndex == 1) // query panel
		{
		    // option to copy to query
		    mi = new JMenuItem("Copy to query");
		    mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			    String val = (String)results.getValueAt(resultsTable.getSelectedRow(),
				    resultsTable.getSelectedColumn());
			    queryPanel.addToQuery(val);
			}
		    });
		    popup.add(mi);
		}

		popup.show(resultsTable, e.getX(), e.getY());
	    }
	}
    }

    /**
     * Jester constructor comment.
     * @param title java.lang.String
     */
    public Jester(String title, String filename) {
        super(title);
	if (spDoc != null) spDoc = new File(filename);

	addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                exit();
	}});
	setIconImage(getImage(APP_ICON));

	dbc = new DBConnection(this);
	dbc.addObserver(this);

	results = new Results();

        Container content = getContentPane();
        content.setLayout(new BorderLayout());

        // create menu bar
        setJMenuBar(createMenuBar());

        // create tool bar
        // split pane forms the body of the app
        JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        splitPane.setPreferredSize(new Dimension(800, 600));

	// create the toolbar
        JToolBar toolbar = new JToolBar(JToolBar.HORIZONTAL);
	toolbar.setFloatable(false);
        conRB = new JRadioButton("Connection", new ImageIcon(getImage(RED_ICON)), false);
        conRB.setSelectedIcon(new ImageIcon(getImage(GREEN_ICON)));
        conRB.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                connect();
            }
        });
	conRB.setToolTipText("Click to (dis)connect");
        toolbar.add(conRB);

	cancelButton = new JButton("Cancel", new ImageIcon(getImage(STOP_ICON)));
	cancelButton.setEnabled(false);
	cancelButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		cancelQuery();
	    }
	});
	cancelButton.setToolTipText("Click to cancel operation");
	toolbar.add(cancelButton);
        content.add(toolbar, BorderLayout.NORTH);

        // set up top tab
        tabbedPane = new JTabbedPane();
        tabbedPane.addTab("Stored Procedures", makeSPPanel());
	queryPanel = new QueryPanel(this);
        tabbedPane.addTab("Query", queryPanel);
        tabbedPane.addTab("Metadata", makeMDPanel());
        splitPane.setTopComponent(tabbedPane);

        // set up bottom tab
        results = new Results();
        resultsTable = new PrintTable(results);
        resultsTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        resultsTable.getColumnModel().getColumn(0).setPreferredWidth(200);
        resultsTable.setDoubleBuffered(true);
        resultsTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	resultsTable.setColumnSelectionAllowed(true);
	resultsTable.setToolTipText("Right click to copy");
        ListSelectionModel rowSM = resultsTable.getSelectionModel();
        rowSM.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                ListSelectionModel lsm = (ListSelectionModel)e.getSource();

                // only display stored procedure or table info if on
                // correct tab
                if (!lsm.isSelectionEmpty() && (tabbedPane.getSelectedIndex() == 2)) {
                    int row = lsm.getMinSelectionIndex();
                    showDetails((String)results.getValueAt(row, 0),
			(String)results.getValueAt(row, 3),
			(String)results.getValueAt(row, 1));
                }
            }
        });
	resultsTable.addMouseListener(new CopyMenu());
        resultsScroll = new JScrollPane(resultsTable);

	JSplitPane resultsSplit = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        resultsSplit.setTopComponent(resultsScroll);

        // set up status area
        status = new JTextArea(5, 50);
        status.setText("Not connected");
        status.setEditable(false);
        JScrollPane statusScroll = new JScrollPane(status);
        Border b = BorderFactory.createTitledBorder(
	    BorderFactory.createBevelBorder(BevelBorder.RAISED), "Status messages");
        statusScroll.setBorder(b);
	resultsSplit.setBottomComponent(statusScroll);
	resultsSplit.setDividerLocation(200);
	resultsSplit.setOneTouchExpandable(true);
	splitPane.setBottomComponent(resultsSplit);
	splitPane.setDividerLocation(250);

        content.add(splitPane, BorderLayout.CENTER);

	// see if there is any config data
	readConfig();
    }

    public Jester(String title) {
        this(title, null);
    }

    /**
     * Add a field/value pair to the defaults
     * @param field parameter name
     * @param value parameter value
     */
    void addDefaultValue(String field, String value) {
        defaultValues.put(field, value);
    }

    /**
     * Append a message to the status bar
     * @param message the message to append
     */
    void appendStatus(String message) {
        status.append(message);
    }

    /** Connect to the database */
    private void connect() {
        if (dbc.isConnected()) {
            disconnect();
            return;
        }

        setCursor(new Cursor(Cursor.WAIT_CURSOR));

        ConnectionDialog cd = new ConnectionDialog(this, currentConnection);
        currentConnection = cd.getConnectionSetting();
        cd.dispose();

        if (currentConnection != null) {
            dbc.connect(currentConnection);

            if (dbc.isConnected()) {
                conRB.setSelected(true);
		filter.setText(currentConnection.dbName.toUpperCase());
		if (currentConnection.spDefFile != null) {
		    loadSPDefs(currentConnection.spDefFile);
		}
            } else {
                conRB.setSelected(false);
            }
        } else {
            conRB.setSelected(false);
        }

        setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }

    /**
     * Create the menu bar
     * @return a menu bar
     */
    private JMenuBar createMenuBar() {
        JMenuBar mb = new JMenuBar();

        // File menu
        JMenu fileMenu = new JMenu("File");
        fileMenu.setMnemonic(KeyEvent.VK_F);

	JMenuItem loadMI = new JMenuItem("Load SP definitions...", KeyEvent.VK_L);
	loadMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.CTRL_MASK));
	loadMI.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		spDoc = null;
		loadSPDefs();
	    }
	});
	fileMenu.add(loadMI);

	JMenuItem saveMI = new JMenuItem("Save SP definitions...", KeyEvent.VK_S);
	saveMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
	saveMI.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		saveSPDefs(true);
	    }
	});
	fileMenu.add(saveMI);

	fileMenu.addSeparator();

	csvMI = new JMenuItem("Save data as CSV...", KeyEvent.VK_V);
	csvMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, ActionEvent.CTRL_MASK | ActionEvent.ALT_MASK));
	csvMI.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		saveAsCSV();
	    }
	});
	fileMenu.add(csvMI);

	fileMenu.addSeparator();

	JMenuItem printMI = new JMenuItem("Print...", KeyEvent.VK_P);
        printMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, ActionEvent.CTRL_MASK));
        printMI.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                printTable();
            }
        });
        printMI.setToolTipText("Print the results table");
        fileMenu.add(printMI);

        fileMenu.addSeparator();

        JMenuItem exitMI = new JMenuItem("Exit", KeyEvent.VK_E);
        exitMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, ActionEvent.CTRL_MASK));
        exitMI.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                exit();
	    }
	});
	exitMI.setToolTipText("Disconnect and exit the application");
	fileMenu.add(exitMI);
	mb.add(fileMenu);

	JMenu dbMenu = new JMenu("Database");
	dbMenu.setMnemonic(KeyEvent.VK_D);

        connectMI = new JMenuItem("Connect...", KeyEvent.VK_C);
        connectMI.setToolTipText("Connect to the database");
        connectMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK | ActionEvent.ALT_MASK));
        connectMI.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                connect();
            }
        });
        dbMenu.add(connectMI);

        disconnectMI = new JMenuItem("Disconnect", KeyEvent.VK_D);
        disconnectMI.setToolTipText("Disconnect from the database");
        disconnectMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.CTRL_MASK));
        disconnectMI.setEnabled(false);
        disconnectMI.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                disconnect();
            }
        });
        dbMenu.add(disconnectMI);

        dbMenu.addSeparator();

	getSPMI = new JMenuItem("Get stored procedures", KeyEvent.VK_G);
	getSPMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_G, ActionEvent.CTRL_MASK));
	getSPMI.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		getStoredProcedures();
	    }
	});
	getSPMI.setEnabled(false);
	dbMenu.add(getSPMI);

	mb.add(dbMenu);

	// Preferences menu
	JMenu prefMenu = new JMenu("Preferences");
	prefMenu.setMnemonic(KeyEvent.VK_O);

	dateMenu = new JMenu("Date");
	dateMenu.setMnemonic(KeyEvent.VK_D);
	JRadioButtonMenuItem ukDFMenu = new JRadioButtonMenuItem("dd/MM/yyyy", true);
	ukDFMenu.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		setDateFormat("dd/MM/yyyy");
	    }
	});
	JRadioButtonMenuItem usDFMenu = new JRadioButtonMenuItem("MM/dd/yyyy");
	usDFMenu.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		setDateFormat("MM/dd/yyyy");
	    }
	});
	JRadioButtonMenuItem spaceDFMenu = new JRadioButtonMenuItem("dd MMM yyyy");
	spaceDFMenu.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		setDateFormat("dd MMM yyyy");
	    }
	});
	ButtonGroup dateBG = new ButtonGroup();
	dateBG.add(ukDFMenu);
	dateBG.add(usDFMenu);
	dateBG.add(spaceDFMenu);
	dateMenu.add(ukDFMenu);
	dateMenu.add(usDFMenu);
	dateMenu.add(spaceDFMenu);
	prefMenu.add(dateMenu);

	timeMenu = new JMenu("Time");
	timeMenu.setMnemonic(KeyEvent.VK_T);
	JRadioButtonMenuItem ukTDMenu = new JRadioButtonMenuItem("HH:mm", true);
	ukTDMenu.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		setTimeFormat("HH:mm");
	    }
	});
	JRadioButtonMenuItem secTDMenu = new JRadioButtonMenuItem("HH:mm:ss");
	secTDMenu.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		setTimeFormat("HH:mm:ss");
	    }
	});
	ButtonGroup timeFormatBG = new ButtonGroup();
	timeFormatBG.add(ukTDMenu);
	timeFormatBG.add(secTDMenu);
	timeFormatBG.add(ukTDMenu);
	timeFormatBG.add(secTDMenu);
	timeMenu.add(ukTDMenu);
	timeMenu.add(secTDMenu);
	prefMenu.add(timeMenu);

	SecondListener secListener = new SecondListener();
	timeoutMenu = new JMenu("Query timeout");
	timeoutMenu.setMnemonic(KeyEvent.VK_T);

	JRadioButtonMenuItem fivesMI = new JRadioButtonMenuItem("5s");
	fivesMI.addActionListener(secListener);

	JRadioButtonMenuItem tensMI = new JRadioButtonMenuItem("10s");
	tensMI.setSelected(true);
	tensMI.addActionListener(secListener);

	JRadioButtonMenuItem thirtysMI = new JRadioButtonMenuItem("30s");
	thirtysMI.addActionListener(secListener);

	JRadioButtonMenuItem onemMI = new JRadioButtonMenuItem("60s");
	onemMI.addActionListener(secListener);

	JRadioButtonMenuItem twomMI = new JRadioButtonMenuItem("120s");
	twomMI.addActionListener(secListener);

	JRadioButtonMenuItem fivemMI = new JRadioButtonMenuItem("300s");
	fivemMI.addActionListener(secListener);

	ButtonGroup timeoutBG = new ButtonGroup();
	timeoutBG.add(fivesMI);
	timeoutBG.add(tensMI);
	timeoutBG.add(thirtysMI);
	timeoutBG.add(onemMI);
	timeoutBG.add(twomMI);
	timeoutBG.add(fivemMI);
	timeoutMenu.add(fivesMI);
	timeoutMenu.add(tensMI);
	timeoutMenu.add(thirtysMI);
	timeoutMenu.add(onemMI);
	timeoutMenu.add(twomMI);
	timeoutMenu.add(fivemMI);
	prefMenu.add(timeoutMenu);

	mb.add(prefMenu);

	// Help menu
	JMenu helpMenu = new JMenu("Help");
	helpMenu.setMnemonic(KeyEvent.VK_H);

	JMenuItem helpMI = new JMenuItem("Help", KeyEvent.VK_H);
	helpMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_H, ActionEvent.CTRL_MASK));
	helpMI.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		showHelp();
	    }
	});
	helpMenu.add(helpMI);

	JMenuItem aboutMI = new JMenuItem("About", KeyEvent.VK_A);
	aboutMI.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		showAbout();
	    }
	});
	helpMenu.add(aboutMI);
	mb.add(Box.createHorizontalGlue());
	mb.add(helpMenu);

	return mb;
    }

    /**
     * Disconnect from the database
     */
    private void disconnect() {
        dbc.disconnect();
        conRB.setSelected(false);
	filter.setText(null);
    }

    /**
     * Exit the program after saving configuration
     */
    private void exit() {
        disconnect();
        dbc.saveConnections();

	try {
	    File home = new File(System.getProperty("user.home"));
	    File configFile = new File(home, CONFIG_FILE);
	    PrintWriter cp = new PrintWriter(new BufferedWriter(
		new FileWriter(configFile)));

	    // save settings
	    cp.print("dateFormat=");
	    cp.println(dateFormat);
	    cp.print("timeFormat=");
	    cp.println(timeFormat);
	    cp.print("timeout=");
	    cp.println(timeout);

	    cp.close();
	} catch (Exception ex) {
	    ex.printStackTrace();
	}

        System.exit(0);
    }

    /** Read any config data */
    private void readConfig() {
	File home = new File(System.getProperty("user.home"));
	File configFile = new File(home, CONFIG_FILE);
	String line, fileName, val;
	StringTokenizer st;

	if (configFile.exists())
	{
	    try {
		BufferedReader reader = new BufferedReader(new FileReader(configFile));
		while ((line = reader.readLine()) != null)
		{
		    if (line.indexOf('=') > -1) {
			st = new StringTokenizer(line, "=");
			st.nextToken();
			val = st.nextToken().trim();

			if (line.startsWith("dateFormat"))
			{
			    setDateFormat(val);
			}
			else if (line.startsWith("timeFormat"))
			{
			    setTimeFormat(val);
			}
			else if (line.startsWith("timeout"))
			{
			    try {
				int to = Integer.parseInt(val);
				setTimeout(to);
			    } catch (NumberFormatException nfex) {
				System.out.println("Invalid timeout value "+val);
			    }
			}
		    }
		}
	    } catch (IOException ioe) {
		System.out.println("Error reading configuration file "+CONFIG_FILE);
	    } catch (NoSuchElementException nsex) {
		System.out.println("Invalid configuration file "+CONFIG_FILE);
	    }
	}
    }

    /**
     * Release any resources being used
     * @exception java.lang.Throwable The exception description.
     */
    protected void finalize() throws Throwable {
        super.finalize();
        disconnect();
    }

    /**
     * Get a reference to DBConnection
     * @return DBConnection
     */
    public DBConnection getDBConnection() {
        return dbc;
    }

    /**
     * Start the application
     * @param args command line arguments
     */
    public static void main(String[] args)
    {
	// output info
	System.out.println("Jester version "+version+", Copyright (C) 2002 Nick Sydenham");
	System.out.println("Jester comes with ABSOLUTELY NO WARRANTY; for details see doc/GPL.txt.");
	System.out.println("This is free software, and you are welcome to redistribute it");
	System.out.println("under certain conditions.");

        Jester jester;
	if (args.length > 0) jester = new Jester("Java Stored Procedure Tester (Jester)", args[0]);
	else jester = new Jester("Java Stored Procedure Tester (Jester)");
        jester.pack();

	// set the location based on screen res
	Toolkit tk = Toolkit.getDefaultToolkit();
	Dimension screen = tk.getScreenSize();
	int sw = (int)(screen.getWidth() - jester.getWidth()) / 2;
	int sh = (int)(screen.getHeight() - jester.getHeight()) / 2;
	jester.setLocation(sw, sh);
	jester.setVisible(true);
    }

    /**
     * Create a label
     * @param name label name
     * @param gridbag java.awt.GridBagLayout
     * @param gbc java.awt.GridBagConstraints
     * @return javax.swing.JLabel
     */
    static JLabel makeLabel(String name, GridBagLayout gridbag, GridBagConstraints gbc)
    {
        JLabel label = new JLabel(name, JLabel.LEFT);
        gridbag.setConstraints(label, gbc);

        return label;
    }

    /**
     * Make a label
     * @param name label name
     * @parma f the font to use
     * @param gridbag java.awt.GridBagLayout
     * @param gbc java.awt.GridBagConstraints
     * @return javax.swing.JLabel
     */
    static JLabel makeLabel(String name, Font f, GridBagLayout gbl, GridBagConstraints gbc)
    {
	JLabel label = makeLabel(name, gbl, gbc);
	label.setFont(f);
	return label;
    }

    /**
     * Make the meta-data panel
     * @return javax.swing.JPanel
     */
    private JPanel makeMDPanel()
    {
        JPanel mdPanel = new JPanel();
        mdPanel.setLayout(new BorderLayout());

        mdArea = new JTextArea(5, 50);
        mdArea.setEditable(false);
        JScrollPane scrollMD = new JScrollPane(mdArea);
        mdPanel.add(scrollMD, BorderLayout.CENTER);

        // add an option pane
        JPanel optionPanel = new JPanel();

        final JRadioButton tables = new JRadioButton("Tables");
	tables.setSelected(true);
        optionPanel.add(tables);

        final JRadioButton procs = new JRadioButton("Procedures");
        optionPanel.add(procs);

	ButtonGroup bg = new ButtonGroup();
	bg.add(tables);
	bg.add(procs);

        JLabel filterLabel = new JLabel("Schema:");
        optionPanel.add(filterLabel);

        filter = new JTextField(10);
        filter.setToolTipText("Schema name to filter by. Ignored if not supported by database.");
        optionPanel.add(filter);

        JButton action = new JButton("Get Metadata", new ImageIcon(getImage(GO_ICON)));
        action.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                status.setText("Getting metadata");
                if (dbc.isConnected()) {
                    int options = 0;
                    if (tables.isSelected()) options += 2;
                    if (procs.isSelected()) options += 1;
                    getMetaData(new Integer(options), filter.getText());
                }
                else status.setText("Please connect first");
            }});
            optionPanel.add(action);

            mdPanel.add(optionPanel, BorderLayout.SOUTH);

            return mdPanel;
    }

    /**
     * Create the Stored Procedure Panel
     * @return javax.swing.JPanel
     */
    private JSplitPane makeSPPanel() {
        spPanel = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, false);

        final JList spList = new JList(spReader);
	spList.setFont(new Font("SansSerif", Font.PLAIN, 12));
	spList.setToolTipText("Right click SP to see options");
        spList.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if (dbc.isConnected()) {
                    sp = (StoredProcedure)spReader.getElementAt(spList.getSelectedIndex());
                    setupInputs();
                }
                else status.setText("Please connect first!");
        }});

	// create pop-up menu
	final JPopupMenu optMenu = new JPopupMenu("Options");
	JMenuItem edit = new JMenuItem("Edit parameter names", KeyEvent.VK_E);
	edit.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		if (sp != null) editSP();
	    }
	});
	JMenuItem del = new JMenuItem("Delete", KeyEvent.VK_D);
	del.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		if (sp != null) deleteSP();
	    }
	});
	optMenu.add(edit);
	optMenu.add(del);

	spList.addMouseListener(new MouseAdapter() {
	    public void mousePressed(MouseEvent e) {
		if ((e.getModifiers() & InputEvent.BUTTON3_MASK) == InputEvent.BUTTON3_MASK) {
		    optMenu.show(spList, e.getX(), e.getY());
		}
	    }
	});
        JScrollPane scroll = new JScrollPane(spList);

        JPanel panel = new JPanel(new BorderLayout());
        panel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEmptyBorder(), "Stored Procedures"));
        panel.add(scroll, BorderLayout.CENTER);

        spPanel.setLeftComponent(panel);

        JPanel inputPanel = new JPanel();
        inputPanel.add(new JLabel("Select a stored procedure"));
        //	inputPanel.setPreferredSize(new Dimension(300, 75));

        inputScroll = new JScrollPane(inputPanel);
        inputScroll.setBorder(BorderFactory.createEtchedBorder());

        spPanel.setRightComponent(inputScroll);
        spPanel.setDividerLocation(150);

        return spPanel;
    }

    /**
     * Output some details about why the SQLException occurred
     * @param sex the SQL exception
     */
    void printSQLException(SQLException sex) {
        status.setText("Stored procedure call failed: "+sex.getMessage());
        status.append("\nError code: "+sex.getErrorCode());
        status.append("\nSQL state: "+sex.getSQLState());
    }

    /**
     * Print table
     */
    private void printTable() {
        PrinterJob pj = PrinterJob.getPrinterJob();

        PageFormat pf = new PageFormat();
        pf.setOrientation(PageFormat.LANDSCAPE);
        Paper paper = new Paper();
        paper.setSize(595, 842); // A4
        pf.setPaper(paper);
        PageFormat newpf = pj.pageDialog(pf);

        // if user pressed OK
        if (newpf != pf) {
            if (pj.printDialog()) {
                pj.setPrintable(resultsTable, newpf);
                try {
                    pj.print();
                    status.setText("Printed table");
                } catch (Exception pex) {
                    status.setText("Print exception: "+pex.toString());
                }
            }
        }
    }

    /**
     * Set the meta-data text area
     * @param metadatainfo the data to display
     */
    void setMetadata(String metadatainfo) {
        mdArea.setText(metadatainfo);
    }

    /**
     * Set the status display
     * @param message the message to display
     */
    void setStatus(String message) {
        status.setText(message);
    }

    /**
     * Set the data to display in the table
     * @param colNames list of column names
     * @param rows list of lists containing row data
     */
    void setTableData(java.util.List colNames, java.util.List rows) {
        results.setData(colNames, rows);
        resultsScroll.revalidate();
    }

    /**
     * Set up the panel that gets the stored procedure inputs
     */
    void setupInputs() {
        inputScroll.setBorder(
            BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),
                "Inputs for "+sp.getName()));

	inputs = new SPInputPanel(this, sp);
        inputScroll.setViewportView(inputs);

        spPanel.revalidate();
    }

    /**
     * Save the data in the results table as a csv file
     */
    private void saveAsCSV()
    {
	String csv = results.toCSV();

	JFileChooser fc = new JFileChooser();
	int resp = fc.showSaveDialog(this);
	if (resp == JFileChooser.APPROVE_OPTION) {
	    File csvFile = fc.getSelectedFile();
	    try {
		BufferedWriter writer = new BufferedWriter(new FileWriter(csvFile));
		writer.write(csv);
		writer.write("\n");
		writer.close();

		JOptionPane.showMessageDialog(this, "Saved to "+csvFile.getName(),
		    "Saved", JOptionPane.INFORMATION_MESSAGE);
	    } catch (IOException ioex) {
		JOptionPane.showMessageDialog(this, ioex.getMessage(),
		    "Cannot save to "+csvFile.getName(), JOptionPane.ERROR_MESSAGE);
	    }
	}
    }

    /**
     * Show an about dialog box
     */
    private void showAbout() {
        JOptionPane.showMessageDialog(this, "Jester version "+version+"\n"+
            "Copyright 2002 Nick Sydenham <nsydenham@yahoo.co.uk>",
            "About Jester", JOptionPane.PLAIN_MESSAGE,
	    new ImageIcon(getImage(APP_ICON)));
    }

    /**
     * Show the help screen
     */
    private void showHelp() {
        if (helpDialog != null) {
            helpDialog.show();
            return;
        }

        setCursor(new Cursor(Cursor.WAIT_CURSOR));

        helpDialog = new JDialog(this, "Help");
        helpDialog.getContentPane().setLayout(new BorderLayout());

        try {
            URL url = getClass().getResource("/doc/Jester_Help.html");
	    try {
		if (url == null) url = new URL("file:///"+System.getProperty("user.dir")+"/doc/Jester_Help.html");
	    } catch (MalformedURLException ex) {
		setStatus("Cannot load help.");
	    }
            JEditorPane helpText = new JEditorPane();
            helpText.setEditable(false);
            helpText.setContentType("text/html");
            helpText.setPage(url);
            JScrollPane helpScroll = new JScrollPane(helpText);
            helpDialog.getContentPane().add(helpScroll, BorderLayout.CENTER);

            helpDialog.setSize(550, 400);
            helpDialog.setLocationRelativeTo(this);
            helpDialog.show();
        } catch (IOException ioe) {
            status.setText(ioe.toString());
        }

        setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }

    /**
     * The DBConnection object has connected/disconnected
     * @param observable DBConnection
     * @param obj the argument passed to the notifyObservers method
     */
    public void update(Observable observable, Object obj)
    {
	boolean active;
	String arg = (String)obj;

	if (arg.equals("C")) {
	    active = true;
	    setTitle("Jester: Connected to "+currentConnection.dbName);
	} else {
	    active = false;
	    setTitle("Jester");
	}

	connectMI.setEnabled(active);
	disconnectMI.setEnabled(active);
	getSPMI.setEnabled(active);
    }

    /** Load a specific SP definition file */
    private void loadSPDefs(String fileName)
    {
	spDoc = new File(fileName);
	loadSPDefs();
    }

    /** Load stored procedure definitions from a file*/
    private void loadSPDefs()
    {
	if (spDoc == null) {
	    JFileChooser fc = new JFileChooser();
	    int returnVal = fc.showOpenDialog(this);
	    if (returnVal == JFileChooser.APPROVE_OPTION) {
		spDoc = fc.getSelectedFile();
	    } else {
		return;
	    }
	}

	try {
	    // Instantiate a parser
	    XMLReader parser = XMLReaderFactory.createXMLReader("org.apache.xerces.parsers.SAXParser");

	    // Register the content handler
	    parser.setContentHandler(spReader);
	    parser.parse(spDoc.getAbsolutePath());

	    status.setText("Loaded stored procedure definitions from "+spDoc.getAbsolutePath());
	} catch (Exception ex) {
	    status.setText("Error parsing SP definition: "+ex.getMessage());
	}
    }

    /**
     * Save the stored procedure definitions to a file
     * @param showDialog if set to true show the file chooser dialog
     * @return returns true if the file was saved
     */
    boolean saveSPDefs(boolean showDialog)
    {
	StoredProcedure sp;

	if (spReader.getSize() > 0)
	{
	    if (showDialog)
	    {
		// get file
		JFileChooser saveFC = new JFileChooser();
		if (spDoc != null) {
		    saveFC.setSelectedFile(spDoc);
		} else {
		    String userHome = System.getProperty("user.home");
		    saveFC.setSelectedFile(new File("sp.xml"));
		}
		int resp = saveFC.showSaveDialog(this);
		if (resp != JFileChooser.APPROVE_OPTION) return false;
		spDoc = saveFC.getSelectedFile();
	    }

	    try {
		// save SPs
		PrintWriter writer = new PrintWriter(
		    new BufferedWriter(new FileWriter(spDoc)));
		writer.println("<StoredProcedures>");
		java.util.List spList = spReader.getStoredProcedures();
		Iterator it = spList.iterator();
		while (it.hasNext()) {
		    sp = (StoredProcedure)it.next();
		    writer.println(sp.getDetails());
		}
		writer.println("</StoredProcedures>");
		writer.close();

		JOptionPane.showMessageDialog(this,
		    "Saved definitions to "+spDoc.getName(),
		    "Saved", JOptionPane.INFORMATION_MESSAGE);
	    } catch (Exception fex) {
		setStatus("Couldn't save definition file: "+fex.getMessage());
		return false;
	    }
	}
	else
	{
	    JOptionPane.showMessageDialog(this, "No stored procedures to save",
		"No stored procedures", JOptionPane.INFORMATION_MESSAGE);
	    return false;
	}

	return true;
    }

    /**
     * Get the currently selected date format
     * @return time format
     */
    public String getDateFormat() {
	return dateFormat;
    }

    /** Set the date format to the specified format */
    private void setDateFormat(String newDateFormat) {
	JMenuItem mi;

	for (int i=0; i<dateMenu.getItemCount(); i++) {
	    mi = dateMenu.getItem(i);
	    if (newDateFormat.equals(mi.getText())) {
		dateFormat = newDateFormat;
		mi.setSelected(true);
		break;
	    }
	}
    }

    /**
     * Get the currently selected time format
     * @return time format
     */
    public String getTimeFormat() {
	return timeFormat;
    }

    /** Set the time format to the specified format */
    private void setTimeFormat(String newTimeFormat) {
	JMenuItem mi;

	for (int i=0; i<timeMenu.getItemCount(); i++) {
	    mi = timeMenu.getItem(i);
	    if (newTimeFormat.equals(mi.getText())) {
		timeFormat = newTimeFormat;
		mi.setSelected(true);
		break;
	    }
	}
    }

    private void setTimeout(int newTimeout) {
	JMenuItem mi;
	String text, timeVal;

	for (int i=0; i<timeoutMenu.getItemCount(); i++) {
	    mi = timeoutMenu.getItem(i);
	    text = mi.getText();
	    timeVal = text.substring(0, text.length()-1);
	    if (newTimeout == Integer.parseInt(timeVal)) {
		timeout = newTimeout;
		mi.setSelected(true);
		break;
	    }
	}
    }

    /**
     * Get the timeout value
     * @return timeout used for queries
     */
    public int getTimeout() {
	return timeout;
    }

    /**
     * Edit the stored procedure input and output parameter names
     */
    void editSP()
    {
	inputScroll.setBorder(
            BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),
                "Editing inputs for "+sp.getName()));

	JPanel paramNames = new ParameterNamesPanel(this, sp);
        inputScroll.setViewportView(paramNames);

        spPanel.revalidate();
    }

    /** Delete a stored procedure */
    void deleteSP()
    {
	int resp = JOptionPane.showConfirmDialog(this, "Really delete "+sp.getName()+"?",
	    "Confirm delete", JOptionPane.YES_NO_OPTION);
	if (resp == JOptionPane.YES_OPTION) spReader.removeStoredProcedure(sp);
    }

    /**
     * Get a default value for a parameter. A default value is one
     * that has been previously entered.
     * @param paramDescription parameter description
     * @return a default value or null
     */
    public String getDefaultParameterValue(String paramDescription) {
	return (String)defaultValues.get(paramDescription);
    }

    /**
     * Load an image from either the JAR file or local directory
     * @param fileName image location
     */
    Image getImage(String fileName)
    {
	URL url = getClass().getResource(fileName);
	try {
	    if (url == null) url = new URL("file:///"+System.getProperty("user.dir")+"/"+fileName);
	} catch (MalformedURLException ex) {
	    setStatus("Cannot load image "+fileName);
	    return null;
	}
	return Toolkit.getDefaultToolkit().createImage(url);
    }

    void callSP(StoredProcedure sp, String[] values)
    {
	Object[] args = {sp, values};
	startQuery("callSP", args);
    }

    void getMetaData(Integer options, String schema)
    {
	Object[] args = {options, schema};
	startQuery("getMetaData", args);
    }

    void performQuery(String query)
    {
	if (dbc.isConnected()) {
	    setCursor(new Cursor(Cursor.WAIT_CURSOR));
	    Object[] args = {query};
	    startQuery("performQuery", args);
	    setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	} else status.setText("Please connect first");
    }

    void showDetails(String type, String schema, String name)
    {
	Object[] args = {type, schema, name};
	startQuery("showDetails", args);
    }

    void getStoredProcedures()
    {
	Object[] args = {spReader, currentConnection.dbName.toUpperCase()};
	startQuery("getStoredProcedures", args);
    }

    /**
     * Start the query
     */
    private void startQuery(String methodName, Object[] args) {
	if (runningQuery) {
	    setStatus("You are already running a query. Either cancel it or "+
		"wait til it finishes.");
	    return;
	}

	dbc.setRunParameters(methodName, args);
	cancelButton.setEnabled(true);
	queryThread = new Thread(dbc);
	queryThread.start();
	runningQuery = true;
    }

    /** Cancel the running query */
    private void cancelQuery()
    {
	queryThread.interrupt();
	cancelButton.setEnabled(false);
	runningQuery = false;
    }

    /** The query has finished */
    void finishedQuery() {
	cancelButton.setEnabled(false);
	runningQuery = false;
    }
}
