package nrs.jester;

import java.awt.*;
import java.awt.print.*;
import java.util.*;
import javax.swing.*;

/*
 * Java Stored Procedure Tester (Jester)
 * Copyright (C) 2002 Nick Sydenham <nsydenham@yahoo.co.uk>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/**
 * A JTable with simplistic printing capability
 *
 * @author Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class PrintTable extends JTable implements Printable
{
    /**
     * PrintTable constructor comment.
     */
    public PrintTable ()
    {
	super ();
    }
    
    /**
     * PrintTable constructor comment.
     * @param rowData java.lang.Object[][]
     * @param columnNames java.lang.Object[]
     */
    public PrintTable (java.lang.Object[][] rowData, java.lang.Object[] columnNames)
    {
	super (rowData, columnNames);
    }
    
    /**
     * PrintTable constructor comment.
     * @param numRows int
     * @param numColumns int
     */
    public PrintTable (int numRows, int numColumns)
    {
	super (numRows, numColumns);
    }
    /**
     * PrintTable constructor comment.
     * @param rowData java.util.Vector
     * @param columnNames java.util.Vector
     */
    public PrintTable (java.util.Vector rowData, java.util.Vector columnNames)
    {
	super (rowData, columnNames);
    }
    /**
     * PrintTable constructor comment.
     * @param dm javax.swing.table.TableModel
     */
    public PrintTable (javax.swing.table.TableModel dm)
    {
	super (dm);
    }
    /**
     * PrintTable constructor comment.
     * @param dm javax.swing.table.TableModel
     * @param cm javax.swing.table.TableColumnModel
     */
    public PrintTable (javax.swing.table.TableModel dm, javax.swing.table.TableColumnModel cm)
    {
	super (dm, cm);
    }
    
    /**
     * PrintTable constructor comment.
     * @param dm javax.swing.table.TableModel
     * @param cm javax.swing.table.TableColumnModel
     * @param sm javax.swing.ListSelectionModel
     */
    public PrintTable (javax.swing.table.TableModel dm, javax.swing.table.TableColumnModel cm, javax.swing.ListSelectionModel sm)
    {
	super (dm, cm, sm);
    }
    
    /**
     * Prints the page at the specified index into the specified
     * {@link Graphics} context in the specified
     * format.  A <code>PrinterJob</code> calls the
     * <code>Printable</code> interface to request that a page be
     * rendered into the context specified by
     * <code>graphics</code>.  The format of the page to be drawn is
     * specified by <code>pageFormat</code>.  The zero based index
     * of the requested page is specified by <code>pageIndex</code>.
     * If the requested page does not exist then this method returns
     * NO_SUCH_PAGE; otherwise PAGE_EXISTS is returned.
     * The <code>Graphics</code> class or subclass implements the
     * {@link PrinterGraphics} interface to provide additional
     * information.  If the <code>Printable</code> object
     * aborts the print job then it throws a {@link PrinterException}.
     * @param graphics the context into which the page is drawn
     * @param pageFormat the size and orientation of the page being drawn
     * @param pageIndex the zero based index of the page to be drawn
     * @return PAGE_EXISTS if the page is rendered successfully
     *         or NO_SUCH_PAGE if <code>pageIndex</code> specifies a
     *	       non-existent page.
     * @exception java.awt.print.PrinterException
     *         thrown when the print job is terminated.
     */
    public int print (Graphics graphics, PageFormat pageFormat, int pageIndex) throws PrinterException
    {
	
	if (pageIndex >= 1)
	{
	    return Printable.NO_SUCH_PAGE;
	}
	
	Graphics2D g2 = (Graphics2D)graphics;
	g2.setColor (Color.black);
	int fontHeight = g2.getFontMetrics ().getHeight ();
	int fontDescent = g2.getFontMetrics ().getDescent ();
	
	//leave room for page number
	double pageHeight = pageFormat.getImageableHeight ()-fontHeight;
	double pageWidth = pageFormat.getImageableWidth ();
	double tableWidth = (double)getColumnModel ().getTotalColumnWidth ();
	double scale = 1;
	if (tableWidth >= pageWidth)
	{
	    scale = pageWidth / tableWidth;
	}
	
	double headerHeightOnPage = getTableHeader ().getHeight ()*scale;
	double tableWidthOnPage = tableWidth*scale;
	
	double oneRowHeight = (getRowHeight ()+getRowMargin ())*scale;
	int numRowsOnAPage = (int)((pageHeight-headerHeightOnPage)/oneRowHeight);
	double pageHeightForTable = oneRowHeight*numRowsOnAPage;
	int totalNumPages = (int)Math.ceil (((double)getRowCount ())/numRowsOnAPage);
	if(pageIndex>=totalNumPages)
	{
	    return NO_SUCH_PAGE;
	}
	
	g2.translate (pageFormat.getImageableX (), pageFormat.getImageableY ());
	g2.drawString ("Page: "+(pageIndex+1),(int)pageWidth/2-35,
	(int)(pageHeight+fontHeight-fontDescent));//bottom center
	
	g2.translate (0f, headerHeightOnPage);
	g2.translate (0f, -pageIndex*pageHeightForTable);
	//TODO this next line treats the last page as a full page
	g2.setClip (0, (int)(pageHeightForTable*pageIndex),(int)Math.ceil (tableWidthOnPage),
	(int)Math.ceil (pageHeightForTable));
	
	g2.scale (scale, scale);
	paint (g2);
	g2.scale (1/scale, 1/scale);
	g2.translate (0f, pageIndex*pageHeightForTable);
	g2.translate (0f, -headerHeightOnPage);
	g2.setClip (0, 0,(int) Math.ceil (tableWidthOnPage),
	(int)Math.ceil (headerHeightOnPage));
	g2.scale (scale, scale);
	getTableHeader ().paint (g2);//paint header at top
	
	return Printable.PAGE_EXISTS;
    }
}
