package nrs.jester;

import javax.swing.event.*;
import java.util.*;
import javax.swing.table.*;

/*
 * Java Stored Procedure Tester (Jester)
 * Copyright (C) 2002 Nick Sydenham <nsydenham@yahoo.co.uk>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/**
 * Table model containing the results
 *
 * @author Nicholas Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class Results extends AbstractTableModel
{
    private List rows = new ArrayList(); // a list of lists
    private List columnNames = new ArrayList();
    
    /**
     * Results constructor comment.
     */
    public Results()
    {
	super();
	columnNames.add("No connection");
	List row = new ArrayList();
	row.add("No data");
	rows.add(row);
    }
    
    /**
     * getColumnCount method comment.
     */
    public int getColumnCount()
    {
	return columnNames.size();
    }
    
    /**
     * Get a column name
     * @param col column index starting from 1
     * @return column name
     */
    public String getColumnName(int col)
    {
	return (String)columnNames.get (col);
    }
    
    /**
     * getRowCount method comment.
     */
    public int getRowCount()
    {
	return rows.size();
    }
    
    /**
     * getValueAt method comment.
     */
    public Object getValueAt(int rowIndex, int columnIndex)
    {
	if (rowIndex > -1 && rowIndex < rows.size()) 
	{
	    List row = (List)rows.get(rowIndex);
	    if (row != null)
	    {
		if (columnIndex > -1 && columnIndex < row.size())
		    return row.get(columnIndex);
	    }
	}
	
	return "";
    }
    
    /**
     * Returns true if the cell at <I>rowIndex</I> and <I>columnIndex</I>
     * is editable.  Otherwise, setValueAt() on the cell will not change
     * the value of that cell.
     *
     * @param	rowIndex	the row whose value is to be looked up
     * @param	columnIndex	the column whose value is to be looked up
     * @return	true if the cell is editable.
     * @see #setValueAt
     */
    public boolean isCellEditable(int rowIndex, int columnIndex)
    {
	return false;
    }
    
    /**
     * Change the data that is held by this model
     * @param newColNames new column names
     * @param newRows new data
     */
    public void setData(List newColNames, List newRows)
    {
	columnNames = newColNames;
	rows = newRows;
	
	fireTableStructureChanged();
    }
    
    /**
     * Sets an attribute value for the record in the cell at
     * <I>columnIndex</I> and <I>rowIndex</I>.  <I>aValue</I> is
     * the new value.
     *
     * @param	aValue		 the new value
     * @param	rowIndex	 the row whose value is to be changed
     * @param	columnIndex 	 the column whose value is to be changed
     * @see #getValueAt
     * @see #isCellEditable
     */
    public void setValueAt(Object aValue, int rowIndex, int columnIndex)
    {
	// not settable
    }
    
    /**
     * Return the contents of the table as comma separated values
     * @return CSV string
     */
    public String toCSV()
    {
	StringBuffer csv = new StringBuffer();
	Iterator it, rowit;
	List row;
	
	// column names
	it = columnNames.iterator();
	while (it.hasNext())
	{
	    csv.append((String)it.next());
	    csv.append(",");
	}
	// remove final ,
	csv.setLength(csv.length()-1);
	csv.append("\n");
	
	// data
	it = rows.iterator();
	while (it.hasNext())
	{
	    row = (List)it.next();
	    rowit = row.iterator();
	    while (rowit.hasNext())
	    {
		csv.append((String)rowit.next());
		csv.append(",");
	    }
	    csv.setLength(csv.length()-1);
	    csv.append("\n");
	}
	
	return csv.toString();
    }
}
