package nrs.jester.db;

import java.sql.*;
import java.util.Properties;

/*
 * Java Stored Procedure Tester (Jester)
 * Copyright (C) 2002 Nick Sydenham <nsydenham@yahoo.co.uk>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/**
 * Factory class which generates a call class to deal with database
 * specific CallableStatements.
 *
 * @author Nicholas Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class CallFactory 
{
    public final static int AS400 = 0;
    public final static int DB2 = 1;
    public final static int FIREBIRD = 2;
    public final static int ORACLE_OCI = 3;
    public final static int ORACLE_THIN = 4;
    public final static int SYBASE_4 = 5;
    public final static int SYBASE_5 = 6;
    
    /** Constants used for driver property names */
    public final static String PROP_DRIVER = "driver";
    public final static String PROP_URL = "url";
    
    private final static String connTypes[] = {
	"AS/400",
	"DB2",
	"Firebird/Interbase InterClient",
	"Oracle OCI",
	"Oracle Thin",
	"Sybase jConnect 4.x",
	"Sybase jConnect 5.x"
    };
    
    private final static String connImpls[] = {
        "nrs.jester.db.AS400Call",
	"nrs.jester.db.DB2Call",
        "nrs.jester.db.FirebirdCall",
        "nrs.jester.db.OracleOCICall",
	"nrs.jester.db.OracleThinCall",
	"nrs.jester.db.Sybase4Call",
	"nrs.jester.db.Sybase5Call"
    };
    
    /**
     * Create an implementation of the DBCall which is specific to the
     * database being used.
     * @param dbType an integer representing the type of database connection
     * @return call instance or null if it can't be created
     */
    public static DBCall createCall(int dbType)
    {
        try {
            Class c = Class.forName(connImpls[dbType]);
            return (DBCall)c.newInstance();
        } catch (Exception ex) {}
        
        return null;
    }

    public static String[] getConnectionProperties(int dbType)
    {
	try {
            Class c = Class.forName(connImpls[dbType]);
            DBCall dbc = (DBCall)c.newInstance();
	    return dbc.getConnectionProperties();
        } catch (Exception ex) {}
	
	return new String[0];
    }

    /**
     * Get a list of supported connection types
     * @return array of connection names
     */
    public static String[] getConnectionTypes()
    {
	return connTypes;
    }
}
