package nrs.jester.db;

import java.util.*;
import java.sql.*;

import nrs.jester.ConnectionSetting;

/*
 * Java Stored Procedure Tester (Jester)
 * Copyright (C) 2002 Nick Sydenham <nsydenham@yahoo.co.uk>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/**
 * Firebird/Interbase InterClient specific instance of DBCall.
 *
 * @author Nicholas Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class FirebirdCall extends DBCall
{
    protected String DRIVER = "interbase.interclient.Driver";
    protected String URL = "jdbc:interbase";
    private final static String[] connProps = {
	"charSet", "roleName", "suggestedCachePages", "sweepOnConnect"
    };
   
     /** Get a Connection to the database
     * @param cs ConnectionSetting
     * @param timeout login timeout
     * @return an SQL connection
     * @exception java.sql.SQLException if a connection can't be made
     */
    public Connection getConnection(ConnectionSetting cs, int timeout) throws SQLException
    {
        try {
	    // either load from CLASSPATH or lib directory
            Class c = Class.forName(DRIVER);
        } catch (ClassNotFoundException e) {
            // A call to Class.forName() forces us to consider this exception :-)...
            throw new SQLException("InterClient JDBC driver not in classpath");
        }

	StringBuffer sb = new StringBuffer(URL);
	sb.append("://");
	sb.append(cs.host);
	if (cs.port > 0) {
	    sb.append(":");
	    sb.append(cs.port);
	}
	sb.append("/");
	sb.append(cs.dbName);
	
	return makeConnection(sb.toString(), cs, 0);
    }
    
    /**
     * Get a ResultSet from the stored procedure call
     * @return java.sql.ResultSet
     */
    public ResultSet getResultSet() throws SQLException {
        return call.getResultSet();
    }
    
    /**
     * Prepare a stored procedure call - output parameters are ignored by InterBase
     * @param conn the connection to the database
     * @param procedure the stored procedure name including whatever package it's in
     * @param inputParams the number of input parameters
     * @param outputTypes the output types
     * @exception SQLException if it goes wrong
     */
    public void prepareCall(Connection conn, String procedure, int inputParams, int[] outputTypes) throws SQLException
    {
        int i;
        
        StringBuffer sql = new StringBuffer("{call ");
        sql.append(procedure);
        
        if( inputParams > 0 ) {
            sql.append(" (");
            
            for (i=0; i<inputParams; i++) {
                sql.append("?,");
            }
            
            if (inputParams>0)
                sql.setLength(sql.length()-1); // remove the last comma
            
            sql.append(")");
        }
        
        sql.append("}");
        
        call = conn.prepareCall(sql.toString());
    }
    
    /**
     * Set the call timeout in seconds. Not supported by Firebird.
     * @param timeout int
     */
    public void setQueryTimeout(int timeout) throws SQLException {}
    
    /**
     * Get optional connection properties
     * @return an array of connection property names
     */
    public String[] getConnectionProperties()
    {
	return connProps;
    }
    
    /**
     * Return true if the DB supports query timeouts.
     * Interclient doesn't support query timeout
     * @return true if query timeout is supported
     */
    public boolean supportsTimeout()
    {
	return false;
    }
}
